import 'dart:io';
import 'package:yona/yona.dart';
import 'dart:math' as math;

var host = NetAddr.fromString("yonafediverse.com:7777");
var n = 10;

void clearConsole() {
    print(Process.runSync("clear", [], runInShell: true).stdout);
}

void main(List<String> arguments) async {
  n = int.parse(arguments[arguments.length - 1]);
  host = NetAddr.fromString(arguments[arguments.length - 2]);
  var user1 = await Crypto.genSK();
  var user2 = await Crypto.genSK();
  var packet = Packet(Crypto.derivePK(user1), Crypto.derivePK(user2),
      await Crypto.randBytes(100), Packet.zeroID);
  var zeroPacket =
      Packet(Crypto.derivePK(user1), Crypto.derivePK(user1), [], Packet.zeroID);
  List<Packet> packets = [zeroPacket];
  var timings = [];
  for (var i = 0; i < n; i++) {
    var startSend = DateTime.now();
    await Longlog.push(host, packet.encode(), user2, Duration(seconds: 2));
    var endSend = DateTime.now();
    var startRecv = DateTime.now();
    packets = await Longlog.pull(
        host,
        zeroPacket.encode().sublist(0, 1 + Packet.pkSize + 8),
        10,
        Duration(seconds: 2));
    var endRecv = DateTime.now();
    assert(packets.length == math.min(i + 1, 10));
    for (var pack in packets) {
      assert(Utils.equal(pack.data, packet.data));
      assert(Utils.equal(pack.from, packet.from));
      assert(Utils.equal(pack.to, packet.to));
    }
    timings.add({
      "send": endSend
          .difference(startSend)
          .inMicroseconds,
      "recv": endRecv
          .difference(startRecv)
          .inMicroseconds,
    });
    clearConsole();
    var progress = (i + 1) / n * 100;
    print("${(progress).toStringAsFixed(2)}%");
    for (var prop in ["send", "recv"]) {
      var min = (timings
          .map((e) => e[prop])
          .reduce((a, b) => math.min(a as int, b as int)) /
          Duration.microsecondsPerSecond as double)
          .toStringAsPrecision(2);
      var max = (timings
          .map((e) => e[prop])
          .reduce((a, b) => math.max(a as int, b as int)) /
          Duration.microsecondsPerSecond as double)
          .toStringAsPrecision(2);
      var avg = (timings.map((e) => e[prop]).reduce((a, b) => a + b) /
          timings.length /
          Duration.microsecondsPerSecond as double)
          .toStringAsPrecision(2);
      print("$prop avg: $avg min: $min max: $max");
    }
  }
  print("done");
}
